/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.model;

import ch.javablog.mobile.trakkcor.util.StringTokenizer;
import java.util.Date;

/**
 *
 * Taken from http://www.nmea.de/nmea0183datensaetze.html#gga:
 * <pre>
 *                                                      11
 *          1         2     3 4        5 6 7  8   9  10  |  12 13  14   15
 *          |         |     | |        | | |  |   |   |  |   | |   |    |
 * $--GGA,hhmmss.ss,llll.ll,a,yyyyy.yy,a,x,xx,x.x,x.x,M,x.x,M,x.x,xxxx*hh<CR><LF>
 * $GPGGA,130305.0,6643.2,N,00833.912,E,1,08,0.94,00499,M,047,M,,*\r\n
 * </pre>
 * <ol>
 * Field Number:
 * <li>1. Universal Time Coordinated (UTC) </li>
 * <li>2.  Latitude</li>
 * <li>3. N or S (North or South)</li>
 * <li>4. Longitude</li>
 * <li>5. E or W (East or West)</li>
 * <li>6. GPS Quality Indicator,
 * 0 - fix not available,
 * 1 - GPS fix,
 * 2 - Differential GPS fix</li>
 * <li>7.  Number of satellites in view, 00 - 12</li>
 * <li>8. Horizontal Dilution of precision</li>
 * <li>9. Antenna Altitude above/below mean-sea-level (geoid)</li>
 * <li>10. Units of antenna altitude, meters</li>
 * <li>11. Geoidal separation, the difference between the WGS-84 earth</li>
 * ellipsoid and mean-sea-level (geoid), "-" means mean-sea-level</li>
 * below ellipsoid
 * <li>12. Units of geoidal separation, meters</li>
 * <li>13. Age of differential GPS data, time in seconds since last SC104
 * type 1 or 9 update, null field when DGPS is not used</li>
 * <li>14. Differential reference station ID, 0000-1023</li>
 * <li>15. Checksum</li>
 * </ol>
 * @author Heiko Maa
 */
public class GPGGAMessage implements NMEAMessage {
    
    private String message;
    
    /** Universal Time Coordinated(UTC) **/
    private  Date utcDate;

    /** latitude in decimal degrees **/
    private  double latitude;
    
    /** longitude in decimal degrees **/  
    private  double longitude;
    
    /**
     *     0 - fix not available,
     *     1 - GPS fix,
     *     2 - Differential GPS fix 
     */
    private  int qualityIndicator;
    
    private  int numberOfSatellites;
    
    /** horizontal dilution of precision **/
    private  double hdop; 
    
    /** antenna altitude above/below sea-level (geoid) */
    private  double altitude;
    
    /** the difference between the WGS-84 earth
     ellipsoid and mean-sea-level (geoid), "-" means mean-sea-level
     below ellipsoid **/
    private double geoidealSeparation;

    
    public void parseMessage(final String message) {
        this.message = message;
        StringTokenizer tokenizer = new StringTokenizer(message,",");

        final String nmeaIdentifier           = tokenizer.nextToken();
        final String tmpUtc                      = tokenizer.nextToken();
        final String tmpNmeaLatitude             = tokenizer.nextToken();
        final String tmpDirectionNorthSouth      = tokenizer.nextToken();
        final String tmpNmeaLongitude            = tokenizer.nextToken();
        final String tmpDirectionEastWest        = tokenizer.nextToken();
        final String tmpQualityIndicator         = tokenizer.nextToken();
        final String tmpNumberOfSatellites       = tokenizer.nextToken();
        final String tmpHdop                     = tokenizer.nextToken();
        final String tmpAltitude                 = tokenizer.nextToken();
        final String tmpUnitAltitude             = tokenizer.nextToken();
        final String tmpGeoidealSeparation       = tokenizer.nextToken();
        final String tmpUnitGeoidealSeparation   = tokenizer.nextToken();
        final String tmpAgeGpsData               = tokenizer.nextToken();
        final String tmpReferenceStationID       = tokenizer.nextToken();
        final String tmpChecksum                 = tokenizer.nextToken();
         
        /**
         * @todo utc date parsing.
         */
        this.utcDate = new Date();
        if (tmpQualityIndicator.equals("0")) {
            // don't parse because no valid signal
            this.message = "";
            return;
        }
        this.latitude = NMEAMessageParsingUtil.parseLatitude(tmpNmeaLatitude, tmpDirectionNorthSouth);
        this.longitude = NMEAMessageParsingUtil.parseLongitude(tmpNmeaLongitude, tmpDirectionEastWest);
        this.altitude = NMEAMessageParsingUtil.parseTolerantDouble(tmpAltitude);
        this.numberOfSatellites = NMEAMessageParsingUtil.parseTolerantInt(tmpNumberOfSatellites);
        this.qualityIndicator = NMEAMessageParsingUtil.parseTolerantInt(tmpQualityIndicator);
        this.hdop = NMEAMessageParsingUtil.parseTolerantDouble(tmpHdop);
        this.geoidealSeparation = NMEAMessageParsingUtil.parseTolerantDouble(tmpGeoidealSeparation);

    }
    
    public Date getUtcDate() {
        return utcDate;
    }

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    public int getQualityIndicator() {
        return qualityIndicator;
    }

    public int getNumberOfSatellites() {
        return numberOfSatellites;
    }

    public double getHdop() {
        return hdop;
    }

    public double getAltitude() {
        return altitude;
    }

    public double getGeoidealSeparation() {
        return geoidealSeparation;
    }
   
    public String getMessage() {
        return this.message;
    }

    public boolean isValid() {
        return qualityIndicator != 0;
    }
    
}
