/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2006  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package ch.javablog.mobile.trakkcor.rms;

import ch.javablog.mobile.trakkcor.model.*;
import ch.javablog.mobile.trakkcor.PlugIn;
import ch.javablog.mobile.trakkcor.PlugInException;
import ch.javablog.mobile.trakkcor.service.TrakkSender;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import net.sf.microlog.Level;
import net.sf.microlog.Logger;

/**
 *
 * @author Heiko Maa
 */
public class Configuration implements PlugIn {
    private final static String RECORD_STORE_NAME = "RS_CONFIG";
    private final static Logger LOG    = Logger.getLogger();
    private final long  serialVersion = 0x007;
    private int bluetoothAccessPeriod = 60;
    private String bluetoothAddress   = "";
    private String url                = "";
    private String id                 = "";
    private String logLevel                  = Level.ERROR_STRING;
    private NMEAMessageType nmeaMessageType  = NMEAMessageType.GPGGA;
    private TrakkSender trakkSender          = null;
    private RecordStore rs                   = null;
 
    /** Creates a new instance of Configuration */
    public Configuration(final TrakkSender trakkSender) {
        this.trakkSender = trakkSender;
    }
    
    public void init() throws PlugInException {
        LOG.debug("init configuration...");
        try {
            // read from rms
            rs = RecordStore.openRecordStore(RECORD_STORE_NAME,true);
            if (rs.getNumRecords() != 0) {
                LOG.info("number of records: " + rs.getNumRecords());
                // read from rms
                final byte[] record = rs.getRecord(1);
                final ByteArrayInputStream bais = new ByteArrayInputStream(record);
                final DataInputStream dis = new DataInputStream(bais);
                final long serialVersion = dis.readLong();
                
                if (serialVersion != this.serialVersion) {
                    LOG.warn("serialVersion is not matching. restauring default");
                    return;
                }
                
                this.bluetoothAddress = dis.readUTF();
                this.url = dis.readUTF();
                this.bluetoothAccessPeriod = dis.readInt();
                this.id = dis.readUTF();
                
                // new since 0x005: trakkSender class (e.g. ch.javablog.mobile.trakkcor.service.impl.HttpPostTrakkSender)
                String trakkSenderClazz = dis.readUTF();              
                if (trakkSenderClazz.startsWith("class ")) {
                    trakkSenderClazz = trakkSenderClazz.substring("class ".length(),trakkSenderClazz.length());
                }
                LOG.debug("class: " + trakkSenderClazz);
                this.setTrakkSender((TrakkSender) Class.forName(trakkSenderClazz).newInstance());
                
                // new since 0x006: logLevel (e.g. Level.ERROR_STRING);
                setLogLevel(dis.readUTF());
                // a little hack :( , cause Level objects are not instancable
                if (getLogLevel().equals(Level.ERROR_STRING)) {
                    LOG.setLogLevel(Level.ERROR);
                } else if (getLogLevel().equals(Level.DEBUG_STRING)) {
                    LOG.setLogLevel(Level.DEBUG);
                } else if (getLogLevel().equals(Level.INFO_STRING)) {
                    LOG.setLogLevel(Level.INFO);
                } else if (getLogLevel().equals(Level.WARN_STRING)) {
                    LOG.setLogLevel(Level.WARN);
                }
            
                // new since 0x007
                final String strNmeaMessageType = dis.readUTF();
                this.nmeaMessageType = NMEAMessageType.valueOf(strNmeaMessageType);
                LOG.info("bluetoothAddress: " + bluetoothAddress);
                LOG.info("url: " + url);
                LOG.info("bluetoothAccessPeriod: " + bluetoothAccessPeriod);
                LOG.info("id: " + id);
                LOG.info("trakkSenderClazz: " + trakkSenderClazz);
                LOG.info("loglevel " + this.getLogLevel());
                LOG.info("nmeaMessageType " + this.nmeaMessageType);
     
            }
        } catch (Exception e) {
            throw new PlugInException(e);
        } 
    }
    
    public void close() throws PlugInException {
        LOG.debug("closing configuration...");
        DataOutputStream dos = null;
        ByteArrayOutputStream baos = null;
        try {
            baos = new ByteArrayOutputStream();
            dos  = new DataOutputStream(baos);
            dos.writeLong(serialVersion);
            dos.writeUTF(bluetoothAddress);
            dos.writeUTF(url);
            dos.writeInt(bluetoothAccessPeriod);
            dos.writeUTF(id);
            
            // new since 0x005;
            dos.writeUTF(this.getTrakkSender().getClass().toString());
            
            // new since 0x006;
            dos.writeUTF(this.getLogLevel());
            
            // new since 0x007;
            dos.writeUTF(nmeaMessageType.getPrefix());
            
            dos.flush();
            dos.close();
            final byte[] record = baos.toByteArray();
            
            if (rs.getNumRecords() == 0) {
                // first time ever saved.
                rs.addRecord(record,0,record.length);
            } else {
                // overwrite previous entry.
                rs.setRecord(1,record,0,record.length);
            }
            baos = null;
            dos  = null;
        } catch (RecordStoreException e) {
            throw new PlugInException(e);
        } catch (IOException io) {
            throw new PlugInException(io);
        } finally {
            if (dos != null) {
                try{ 
                    dos.close();
                } catch (IOException io) {
                    LOG.error(io.toString(),io);
                }
            }
            try {
                rs.closeRecordStore();
            } catch (RecordStoreException e) {
                LOG.error(e.toString(),e);
            }
        }   
    }
    
    public void setBluetoothAddress(String bluetoothAddress) {
        this.bluetoothAddress = bluetoothAddress;
    }
    
    public void setUrl(String url) {
        this.url = url;
    }
    /**
     * Sets the bluetooth access period in seconds. That means that every <period>
     * seconds trakkcor will connect to the gps device and get the position data.
     */ 
    public void setBluetoothAccessPeriod(int period) {
        this.bluetoothAccessPeriod = period;
    }

    public void setDeviceId(String id) {
    	this.id = id;
    }
    
    public void setNmeaMessageType(NMEAMessageType nmeaMessageType) {
        this.nmeaMessageType = nmeaMessageType;
    }
    
    /**
     * Gets the bluetooth access period in seconds. 
     */ 
    public String getBluetoothAddress() {
        return bluetoothAddress;
    }

    public String getUrl() {
        if(url == null || url.length() < 1) {
        	return "http://www.";
        }
        return url;
    }
    
    public int getBluetoothAccessPeriod() {
        return this.bluetoothAccessPeriod;
    }
   
    public String getDeviceId() {
    	return this.id;
    }
    
    public NMEAMessageType getNmeaMessageType() {
        return nmeaMessageType;
    }

    public TrakkSender getTrakkSender() {
        return trakkSender;
    }

    public void setTrakkSender(TrakkSender trakkSender) {
        this.trakkSender = trakkSender;
    }

    public String getLogLevel() {
        return logLevel;
    }

    public void setLogLevel(String logLevel) {
        // check if level is valid
        if (!Level.DEBUG_STRING.equals(logLevel) &&
            !Level.ERROR_STRING.equals(logLevel) &&
            !Level.WARN_STRING.equals(logLevel) &&
            !Level.INFO_STRING.equals(logLevel)) {
            throw new IllegalArgumentException("loglevel must be a valid level!" + logLevel + " is not valid.");
        } 
        
        this.logLevel = logLevel;
    }
}
