/*
 *    trakkcor - j2me gps tracking
 *    Copyright (C) 2008  Heiko Maa (trakkcor@javablog.ch)
 * 
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package ch.javablog.mobile.trakkcor.service.impl;

import ch.javablog.mobile.trakkcor.controller.listener.GPSListener;
import ch.javablog.mobile.trakkcor.model.*;
import ch.javablog.mobile.trakkcor.service.GPSReceiver;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import net.sf.microlog.Logger;

/**
 * SimpleGPSReceiver accesses the bluetooth gps receiver and fetches
 * the GPGGA nmea message. The receiving process is perfomed in background.
 * Everytime a nmea message is received, the SimpleGPSReceiver notifies its
 * listeners. 
 * @author Heiko Maa
 */
public class SimpleGPSReceiver implements GPSReceiver {
    private final Vector listenerList;
    private final static Logger LOG = Logger.getLogger();
    private String bluetoothAddress;
    private String message;
    private Timer timer;
    private volatile NMEAMessage nmeaMessage;
    private NMEAMessageType nmeaMessageType;
    /**
     * Creates a new instance of SimpleGPSReceiver
     */
    public SimpleGPSReceiver() {
        this.listenerList = new Vector(2);
    }
    
    public void setBluetoothAddress(final String bluetoothAddress) {
        this.bluetoothAddress = bluetoothAddress;
    }
    
    public void addListener(final GPSListener listener) {
        this.listenerList.addElement(listener);
    }
    
    public void removeListener(final GPSListener listener) {
        this.listenerList.removeElement(listener);
    }
    
    public void notifyListeners() {
        final Enumeration enumListener = listenerList.elements();
        while (enumListener.hasMoreElements()) {
            GPSListener listener = (GPSListener) enumListener.nextElement();
            if (nmeaMessage instanceof GPGGAMessage) {
                listener.GGAReceived((GPGGAMessage)nmeaMessage);
            }
            if (nmeaMessage instanceof GPRMCMessage) {
                listener.GPRMCReceived((GPRMCMessage)nmeaMessage);
            }
        }
    }
    public void start(final long period) {
        if (bluetoothAddress == null) {
            throw new IllegalStateException("Bluetooth address is not set");
        }
        if (timer != null) {
            timer.cancel();
        }
        
        final TimerTask gpsTask = new GPSTimerTask();
        timer = new Timer();
        timer.schedule(gpsTask,0,period);
    }
    
    public void stop() {
        if (timer != null) {
            timer.cancel();
        }
    }
    
    private class GPSTimerTask extends TimerTask {
        public void run() {
            StreamConnection conn = null;
            InputStream is = null;
            int maxAttempts = 5;
            boolean ggaFound = false;
            int attempt = 0;
           // warning: S65 Buffer can only hold 600 Bytes, after that it crashes!!!
           // so I have to open several streams until until a GGA Message appears.
            while (!ggaFound && attempt < maxAttempts) {
                try {
                    attempt++;
                    
                    // open connection to device
                    conn = (StreamConnection) Connector.open("btspp://" + SimpleGPSReceiver.this.bluetoothAddress +":1",Connector.READ);
                    is = conn.openInputStream();
            
                    
                    final byte buffer[] = new byte[256];
                    is.read(buffer);
                    message = new String(buffer);
                    LOG.debug("message : " + message);
                    // for s65 the inputstream must be closed immediately.
                    is.close();
                    conn.close();
                    is = null;
                    conn = null;
                                        
                    // get full message
                    int startIndex = message.indexOf(SimpleGPSReceiver.this.nmeaMessageType.getPrefix());
                    int endIndex   = message.indexOf("\r\n",startIndex);
                    
                    if (startIndex != -1 && startIndex < endIndex) {
                        message = message.substring(startIndex,endIndex);
                        try {
                            SimpleGPSReceiver.this.nmeaMessage =  SimpleGPSReceiver.this.nmeaMessageType.createNMEAMessage(message);
                        } catch (Exception e) {
                            LOG.error(e.toString());
                        }
                        notifyListeners(); 
                        ggaFound = true;
                    } 
                } catch (Exception e) {
                     SimpleGPSReceiver.LOG.error(e.toString(),e);
                } finally {
                  
                    try {
                        if (conn != null) {
                            conn.close();
                        }
                    } catch (Exception ignored) {}
                    try {
                        if (is != null) {
                            is.close();
                        }
                    } catch (Exception ignored) {}
                }
                
            }
        }
    }

    public void setNMEAMessageType(NMEAMessageType nmeaMessageType) {
        this.nmeaMessageType = nmeaMessageType;
    }
}
